package org.example.country.report.actions;

import java.io.File;
import java.io.FileOutputStream;
import java.util.List;

import org.apache.log4j.Logger;
import org.example.country.report.entities.Country;

import com.itextpdf.text.Anchor;
import com.itextpdf.text.BaseColor;
import com.itextpdf.text.Document;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.Element;
import com.itextpdf.text.Font;
import com.itextpdf.text.FontFactory;
import com.itextpdf.text.Image;
import com.itextpdf.text.PageSize;
import com.itextpdf.text.Paragraph;
import com.itextpdf.text.Phrase;
import com.itextpdf.text.pdf.PdfPCell;
import com.itextpdf.text.pdf.PdfPTable;
import com.itextpdf.text.pdf.PdfWriter;

public class PDFBuilderImpl 
	implements PDFBuilder
{
	private static final Logger LOG = Logger.getLogger(PDFBuilderImpl.class); 
	
	private Document document;
	private PdfWriter writer;
	private ConfigurationManager propertyManager;
	private int topMargin = 50;
	private int leftMargin = 50;
	private int rightMargin = 50;
	private int bottomMargin = 50;

	@Override
	public void initialize(ConfigurationManager propertyManager)
	{
		LOG.debug("Initializing PDFBuilder");
		this.propertyManager = propertyManager;
		
		String pdfFolder = propertyManager.getProperty("report.folder");
		String pdfName = propertyManager.getProperty("report.name");
		File report = new File(pdfFolder + File.separator + pdfName);
		LOG.info("Report name: " + report);
		
		try
		{
			LOG.trace("Creating Document");
			document = new Document(PageSize.LETTER, leftMargin, rightMargin, topMargin, bottomMargin);
			LOG.trace("Creating PdfWriter");
			writer = PdfWriter.getInstance(document, new FileOutputStream(report));
			LOG.trace("Opening Document");
			document.open();
		} 
		catch (Exception e)
		{
			LOG.error("Failed to create PDFBuider");
			LOG.error(e.getMessage());
            System.exit(8);
		}
	}

	@Override
	public void outputCounties(String currentContinent, List<Country> countries)
	{
		LOG.debug("Outputting: " + currentContinent);
		try
		{
			LOG.trace("Creating Continent header");
			Paragraph title = new Paragraph(currentContinent,
					FontFactory.getFont(FontFactory.COURIER, 14, Font.BOLD, BaseColor.GRAY));
			document.add(title);
		
			Font font = FontFactory.getFont(FontFactory.COURIER, 14, Font.BOLD, BaseColor.LIGHT_GRAY);
			Font anchorFont = FontFactory.getFont(FontFactory.COURIER, 12, Font.BOLD, BaseColor.LIGHT_GRAY);
			
			LOG.trace("Creating table to hold Contries");
			PdfPTable table = new PdfPTable(3);

			table.setSpacingBefore(25);
			LOG.trace("Adding header cells");
			PdfPCell c1 = new PdfPCell(new Phrase("Name", font));
			table.addCell(c1);
			PdfPCell c2 = new PdfPCell(new Phrase("Code", font));
			table.addCell(c2);
			PdfPCell c3 = new PdfPCell(new Phrase("Capital", font));
			table.addCell(c3);
			
			for (Country currentCountry : countries)
			{
				LOG.trace("Adding Country: " + currentCountry.getName());
				table.addCell(currentCountry.getName());
				table.addCell(currentCountry.getCode());
				table.addCell(currentCountry.getCapital());
			}
			LOG.trace("Adding table to document");
			document.add(table);
		
			Anchor anchor = new Anchor("Back To Top", anchorFont);
			anchor.setReference("#HeaderAnchor");
			
			LOG.trace("Adding Back Tp Top ancher");
			document.add(anchor);
		} 
		catch (DocumentException e)
		{
			LOG.error("Failed add Countries for Continent: " + currentContinent);
			LOG.error(e.getMessage());
            System.exit(8);
		}	
	}
	
	@Override
	public void outputHeader()
	{
		LOG.debug("Adding Document Header");
		try
		{
			Anchor headerAnchor = new Anchor("Countries By Continent",
					FontFactory.getFont(FontFactory.COURIER, 16, Font.BOLD, BaseColor.DARK_GRAY));
			headerAnchor.setName("HeaderAnchor");
			Paragraph headerParagraph = new Paragraph();

			headerParagraph.setSpacingBefore(50);

			LOG.trace("Adding anchor for Top Of Page");
			headerParagraph.add(headerAnchor);		
			
			headerParagraph.setIndentationLeft(80);
			headerParagraph.setIndentationRight(80);
			headerParagraph.setAlignment(Element.ALIGN_CENTER);
			headerParagraph.setSpacingAfter(15);
			
			LOG.trace("Adding Header to Document");
			document.add(headerParagraph);
			
			String resourceFolder = propertyManager.getProperty("resource.folder");
			String resourceLogo = propertyManager.getProperty("resource.logo");
			String logo = resourceFolder + File.separator + resourceLogo;

			Image logoImage = Image.getInstance(logo);
			logoImage.scaleAbsolute(240f, 120f);
			
			Paragraph logoParagraph = new Paragraph();
			logoParagraph.add(logoImage);
			logoParagraph.setAlignment(Element.ALIGN_LEFT);
			
			LOG.trace("Adding Logo image");
			document.add(logoParagraph);
		} 
		catch (Exception e)
		{
			 LOG.error("Failed to create PDF");
	         LOG.error(e.getMessage());
	         System.exit(8);
		}
	}

	@Override
	public void complete()
	{
		LOG.debug("Completing Document");
		document.close();
		writer.close();
	}
}
