package org.example.country.report.entities;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.log4j.Logger;

/**
 * This is the entity class to hold the Country data from DB2.
 * <br>
 * This class hold the results of this SQL:
 * <pre>
 * SELECT country.code as Code, country.name as Name,
 *   country.continent as Continent, city.name as Capital 
 *	 FROM jearl.countries AS country 
 *   JOIN jearl.cities AS city 
 *   ON country.capital = city.id 
 *	 WHERE country.continent = ? 
 *	 ORDER BY Name
 * </pre>
 * 
 * @author Jonathan Earl
 * @version 2.0
 * @since 1.0
 *
 */
public class Country implements Serializable
{
	private static final long serialVersionUID = 1L;
	private static final Logger LOG = Logger.getLogger(Country.class); 
	
	private static List<String> continents;

	private String code;
	private String name;
	private String capital;
	private String continent;
	
	// this static block will run only once
	//  when this class is loaded into memory
	static
	{
		continents = new ArrayList<String>();
		continents.add("Africa");
		continents.add("Antarctica");
		continents.add("Asia");
		continents.add("Europe");
		continents.add("North America");
		continents.add("Oceania");
		continents.add("South America");  
	}

	/**
	 * Constructing a default Country.
	 * <br>
	 * The initial values are:
	 * <ul>
	 *   <li><strong>code</strong>: ABW</li>
	 *   <li><strong>name</strong>: Aruba</li>
	 *   <li><strong>capital</strong>: Oranjestad</li>
	 *   <li><strong>continent</strong>: North America</li>
	 * </ul>
	 */
	public Country()
	{ 
		LOG.debug("Constructing a Country object");
		setCode("ABW");
		setName("Aruba");
		setCapital("Oranjestad");
		setContinent("North America");
	}

	/**
	 * This return the Country Code.
	 * 
	 * @return the Country code
	 */
	public String getCode()
	{
		LOG.debug("Getting code: " + code);
		return code;
	}

	/**
	 * This set the Country Code.
	 * <br> 
	 * The rules are:
	 * <ul>
	 *   <li>The code must <strong>not</strong> null.</li>
	 *   <li>The code must <strong>not</strong> empty.</li>
	 *   <li>The code must <strong>be</strong> 3 chars in length.</li>
	 *   <li>The code will be converted to upper case.</li>
	 * </ul>
	 * 
	 * @param code the code input value
	 * @throws IllegalArgumentException if the input code is invalid
	 */
	public void setCode(String code)
	{
		LOG.debug("Setting code: " + code);
		if (code == null)
		{
			throw new IllegalArgumentException("code must not be null");
		}
		code = code.trim();
		if (code.isEmpty())
		{
			throw new IllegalArgumentException("code must not be empty");
		}
		if (code.length() != 3)
		{
			throw new IllegalArgumentException("code must be 3 chars in length");
		}
		this.code = code;
	}

	/**
	 * This return the Country Name.
	 * 
	 * @return the Country name
	 */
	public String getName()
	{
		LOG.debug("Getting name: " + name);
		return name;
	}

	/**
	 * This set the Country Name.
	 * <br> 
	 * The rules are:
	 * <ul>
	 *   <li>The name must <strong>not</strong> null.</li>
	 *   <li>The name must <strong>not</strong> empty.</li>
	 *   <li>The name must be 52 chars or less in length.</li>
	 * </ul>
	 * 
	 * @param name input value to set
	 * @throws IllegalArgumentException if the input name is invalid
	 */
	public void setName(String name)
	{
		LOG.debug("Setting name: " + name);
		if (name == null)
		{
			throw new IllegalArgumentException("name must not be null");
		}
		name = name.trim();
		if (name.isEmpty())
		{
			throw new IllegalArgumentException("name must not be empty");
		}
		if (name.length() > 52)
		{
			throw new IllegalArgumentException("name must be 52 chars or less in length");
		}
		this.name = name;
	}

	/**
	 * This return the Country Capital.
	 * 
	 * @return the Country capital
	 */
	public String getCapital()
	{
		LOG.debug("Getting capital: " + capital);
		return capital;
	}

	/**
	 * This set the Country Capital.
	 * <br> 
	 * The rules are:
	 * <ul>
	 *   <li>The capital must <strong>not</strong> null.</li>
	 *   <li>The capital must <strong>not</strong> empty.</li>
	 *   <li>The capital must be 35 chars or less in length.</li>
	 * </ul>
	 * 
	 * @param capital input value to set
	 * @throws IllegalArgumentException if the input capital is invalid
	 */
	public void setCapital(String capital)
	{
		LOG.debug("Setting capital: " + capital);
		if (capital == null)
		{
			throw new IllegalArgumentException("capital must not be null");
		}
		capital = capital.trim();
		if (capital.isEmpty())
		{
			throw new IllegalArgumentException("capital must not be empty");
		}
		if (capital.length() > 35)
		{
			throw new IllegalArgumentException("capital must be 35 chars or less in length");
		}
		this.capital = capital;
	}

	/**
	 * This return the Country Continent.
	 * 
	 * @return the Country continent
	 */
	public String getContinent()
	{
		LOG.debug("Getting continent: " + continent);
		return continent;
	}

	/**
	 * This set the Country Continent.
	 * <br> 
	 * The rules are:
	 * <ul>
	 *   <li>The continent must <strong>not</strong> null.</li>
	 *   <li>The continent must <strong>not</strong> empty.</li>
	 *   <li>The continent must be 20 chars or less in length.</li>
	 *   <li>The continent must be in this list:
	 *       <ul>
	 *         <li>Africa</li>
	 *         <li>Antarctica</li>
	 *         <li>Asia</li>
	 *         <li>Europe</li>
	 *         <li>North America</li>
	 *         <li>Oceania</li>
	 *         <li>South America</li>
	 *       </ul>
	 *   </li>
	 * </ul>
	 * 
	 * @param continent input value to set
	 * @throws IllegalArgumentException if the input continent is invalid
	 */
	public void setContinent(String continent)
	{
		LOG.debug("Setting continent: " + continent);
		if (continent == null)
		{
			throw new IllegalArgumentException("continent must not be null");
		}
		continent = continent.trim();
		if (continent.isEmpty())
		{
			throw new IllegalArgumentException("continent must not be empty");
		}
		if (continent.length() > 20)
		{
			throw new IllegalArgumentException("continent must be 20 chars or less in length");
		}
		if (!continents.contains(continent))
		{
			throw new IllegalArgumentException("continent must be a valid name");
		}
		this.continent = continent;
	}

	/**
	 * This is the toString for the Country class.
	 * <br>
	 * It returns:<br>
	 * Country [code=xxx, name=xxx, capital=xxx,
	 *     continent=xxx]
	 * 
	 * @return the Country object as a String.
	 * 
	 */
	@Override
	public String toString()
	{
		return "Country [code=" + code + ", name=" + name + ", capital="
				+ capital + ", continent=" + continent + "]";
	}

	/**
	 * This is the hashCode method for the Country class.
	 * <br>
	 * It uses:
	 * <ul>
	 *   <li>code</li>
	 *   <li>name</li>
	 *   <li>capital</li>
	 *   <li>continent</li>
	 * </ul>
	 * 
	 * @return the generated hashCode
	 */
	public int hashCode(){
		return new HashCodeBuilder()
		.append(getCode())
		.append(getName())
		.append(getCapital())
		.append(getContinent())
		.toHashCode();
	}	
	
	/**
	 * This is the equals method for the Country class.
	 * <br>
	 * It uses:
	 * <ul>
	 *   <li>code</li>
	 *   <li>name</li>
	 *   <li>capital</li>
	 *   <li>continent</li>
	 * </ul>
	 * 
	 * @return true of all the compared fields are equal
	 */
	@Override
	public boolean equals(final Object obj) 
	{
		if(obj instanceof Country) {
			final Country other = (Country) obj;
			return new EqualsBuilder()
			.append(getCode(), other.getCode())
			.append(getName(), other.getName())
			.append(getCapital(), other.getCapital())
			.append(getContinent(), other.getContinent())
			.isEquals();
		} else {
			return false;
		}
	}
}
